"""
This module provides orchestration to run and execute Python Jobs from the command line
"""
from re import X
from typing import Callable, Any, List
from jobtools.arguments import TaskArguments, get_args_from_signature, get_parser_from_signature

class TaskRunner():
    """
    Allows an easy way to run a task in Azure ML and log any input into the run. Tasks are
    specifiedas a callable with arguments that are automatically parsed from the signature. The
    method should return a dictionary with keys "metrics", "arguments" and "artifacts".
    """
    def __init__(self, args: TaskArguments = None, ignore_arguments: List[str] = []) -> None:
        """
        Initializes the TaskRunner. If `args` is indicated, then the argument's won't be parsed
        from the command line. Otherwise they will.
        """
        self.task_arguments = args
        self.ignore_arguments = ignore_arguments

    def run(self, task: Callable[[], Any]) -> Any:
        """
        Runs the given task specified as a method. Arguments are determined from the signature of
        the method and they should match the arguments indicated in the command line unless they
        are indicated using the `args` arguments during `init`.

        Parameters
        ----------
        task: Callable[[], Dict[TaskResultKey, Dict[str, Any]]]
            The task you want to run. This method can be any that returns a dictionary with keys
            `arguments`, `metrics` and `artifacts`. Parameters for this method will either be
            demanded from the command line or have to be indicated in the constructor of the
            `TaskRunner` object.
        """
        if self.task_arguments is None:
            args = get_args_from_signature(task, self.ignore_arguments)
        else:
            args = self.task_arguments.resolve_for_method(task)

        return task(**args)
    
    def display_help(self, task: Callable[[], Any]) -> None:
        parser = get_parser_from_signature(task, self.ignore_arguments)
        parser.print_help()



import argparse
from ctypes import ArgumentError
import sys
from pathlib import Path
from importlib.machinery import SourceFileLoader
from jobtools.runner import TaskRunner

if __name__ == "__main__":
    if len(sys.argv) >= 3:
        module_path = sys.argv[1]
        method = sys.argv[2]
    else:
        parser = argparse.ArgumentParser('pyrunit')
        parser.add_argument("file.py", help='path to the Python source file')
        parser.add_argument("MyTask", help='name of the function to call')
        parser.parse_known_args()
    
    if not Path(module_path).exists():
        raise FileNotFoundError(module_path)

    module_name = Path(module_path).stem
    modulevar = SourceFileLoader(module_name, module_path).load_module()
    callable_func = getattr(modulevar, method)

    tr = TaskRunner(ignore_arguments = sys.argv[1:3])
    tr.run(callable_func)



import setuptools

with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name="jobtools",
    version="0.0.10",
    author="Facundo Santiago",
    description="Facilitates the use of Python from the command line",
    url = 'https://github.com/santiagxf/jobtools',
    long_description=long_description,
    long_description_content_type="text/markdown",
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],                                      
    python_requires='>=3.6',                
    packages=setuptools.find_packages(where='src', exclude=("tests",)),   
    package_dir={'':'src'},  
    scripts=['./src/scripts/pyrunit', './src/scripts/jobtools'],
    include_package_data=True,   
    install_requires=[
        'PyYAML',
    ]
)